/* SPDX-License-Identifier: BSD-3-Clause */

#ifndef __SO_RINGBUFFER_H__
#define __SO_RINGBUFFER_H__

#include <sys/types.h>
#include <string.h>
#include <pthread.h>

typedef struct so_ring_buffer_t {
	char *data;
	size_t read_pos;
	size_t write_pos;
	size_t len;
	size_t cap;
	/* TODO: Add syncronization primitives */
	pthread_mutex_t mutex; // mutex for synchronization access to the buffer
	pthread_cond_t not_empty; // condition variable for signaling when the buffer is not empty
	pthread_cond_t not_full;// condition variable for signaling when the buffer is not full
	int stop; // flag to signal that the buffer is stopped
} so_ring_buffer_t;

int     ring_buffer_init(so_ring_buffer_t *rb, size_t cap);
ssize_t ring_buffer_enqueue(so_ring_buffer_t *rb, void *data, size_t size);
ssize_t ring_buffer_dequeue(so_ring_buffer_t *rb, void *data, size_t size);
void    ring_buffer_destroy(so_ring_buffer_t *rb);
void    ring_buffer_stop(so_ring_buffer_t *rb);

#endif /* __SO_RINGBUFFER_H__ */
